precision mediump float;

varying vec2      v_texCoord2D;

uniform sampler2D u_texture2D;
uniform vec2      u_lensPP;
uniform vec2      u_lensD;
uniform float     u_ratio;
uniform float     u_viewRatio; // ratio of view port
uniform float     u_zoneStroke;
uniform vec4      u_zoneInfo; //  x, y, z, w = (x, y) (h, w)
uniform int       u_drawzone;
uniform int       u_support_3d;   // 1 or 0

bool isOnBorder(float originX, float originY, float measuredX, float measuredY, float x_half_edge, float y_half_edge, float x_stroke, float y_stroke) {
    return ((originX - x_half_edge - y_stroke <= measuredX && measuredX <= originX + x_half_edge + y_stroke)
           && ((originY - y_half_edge - x_stroke <= measuredY && measuredY <= originY - y_half_edge)
            || (originY + y_half_edge <= measuredY && measuredY <= originY + y_half_edge + x_stroke)));
}

void main () {
    vec2 uv = v_texCoord2D;
    if (u_support_3d == 0) {
        if(u_drawzone == 1) {
            // viewport(w / h != 1), then fix the lens && strokes
            bool isOnBorderX = isOnBorder(u_zoneInfo.x, u_zoneInfo.y, uv.x, uv.y, u_zoneInfo.w / u_viewRatio, u_zoneInfo.z, u_zoneStroke * u_viewRatio, u_zoneStroke);
            bool isOnBorderY = isOnBorder(u_zoneInfo.y, u_zoneInfo.x, uv.y, uv.x, u_zoneInfo.z, u_zoneInfo.w / u_viewRatio, u_zoneStroke, u_zoneStroke * u_viewRatio);
            if(isOnBorderX || isOnBorderY) {
                // 0x02d4cd : 02, 212, 204 : vec4(r/255, g/255, b/255, a);

                // mix-01:
                // gl_FragColor = texture2D(u_texture2D, uv) * vec4(0.007843137254902, 0.831372549019608, 0.803921568627451, 1.0);

                // mix-02: mix(u, v, a) { return u * (1.0 -a) + v * a);
                // gl_FragColor = mix( texture2D(u_texture2D, uv), vec4(0.007843137254902, 0.831372549019608, 0.803921568627451, 1.0), 0.6);

                // without-mix:
                gl_FragColor = vec4(0.007843137254902, 0.831372549019608, 0.803921568627451, 1.0);
           } else {
                gl_FragColor = texture2D(u_texture2D, uv);
           }
        } else {
            gl_FragColor = texture2D(u_texture2D, uv);
        }
    } else {
        // move to center
        uv = vec2(uv.x - 0.5 + u_lensPP.x, uv.y - 0.5 + u_lensPP.y);
        // resize to fit view
        uv = vec2((uv.x - u_lensPP.x) * u_lensD.x * u_ratio + u_lensPP.x,
                  (uv.y - u_lensPP.y) * u_lensD .y + u_lensPP.y);

        // add mask on outside lens view
        vec2 tmp = uv - u_lensPP;
        float r = u_lensD.y / 2.0;
        tmp.x = tmp.x * 1.3333;
        if (tmp.x * tmp.x + tmp.y * tmp.y < r * r) {
            gl_FragColor = texture2D(u_texture2D, uv);
        } else {
//            if ((uv.x < 0.26 && uv.y < 0.06)) {
//                gl_FragColor = texture2D(u_texture2D, uv);
//            }
//            else {
                gl_FragColor = vec4(0.0, 0.0, 0.0, 0.0);
//            }
        }
    }
}